
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# WebDAV can send and receive files.
# 
# We could have intermediate WebDAV servers, or use WebDAV and certificates instead of ssh.
# Intermediate guys could charge money.
#
# 